/*
 * Dictionary: transforming string to integer-id.
 */

#include <config.h>

#include <stdio.h>
#include "Dictionary.h"

Dictionary::Dictionary() {
}

Dictionary::~Dictionary() {
	for (vector<string*>::iterator iter = id2word.begin(); iter != id2word.end(); ++iter) {
		delete *iter;
	}
}

bool Dictionary::load(const char *file) {
	FILE *f = fopen(file, "r");
	if (!f)
		return false;
	char line[MAX_LINE_SIZE];
	errno = 0;
	while (fgets(line, sizeof(line), f)) {
		chomp(line);
		int i = id2word.size() + DICTIONARY_MIN_ID;
		word2id[line] = i;
		id2word.push_back(new string(line));
		errno = 0;
	}
	if (errno > 0)
		return false;
	fclose(f);
	return true;
}

bool Dictionary::save(const char *file) {
	FILE *f = fopen(file, "w");
	if (!f)
		return false;
	for (vector<string*>::iterator iter = id2word.begin(); iter != id2word.end(); ++iter) {
		if (fprintf(f, "%s\n", (*iter)->c_str()) < 0)
			return false;
	}
	fclose(f);
	return true;
}

int Dictionary::getId(const char *str, bool insert) {
	if (!str)
		return DICTIONARY_NULL;
	tr1::unordered_map<string, int>::iterator iter = word2id.find(str);
        if (iter == word2id.end()) {
		if (insert) {
			int i = id2word.size() + DICTIONARY_MIN_ID;
			word2id[str] = i;
			id2word.push_back(new string(str));
			return i;
		}
                return DICTIONARY_UNKNOWN;
	}
        return iter->second;
}

const char *Dictionary::getString(int i) {
	if (i <= DICTIONARY_UNKNOWN || i >= (int)id2word.size() + DICTIONARY_MIN_ID)
		return "NA";
	if (i == DICTIONARY_NULL)
		return "NULL";
	return id2word[i-DICTIONARY_MIN_ID]->c_str();
}

string *Dictionary::decodeVector(vector<int> *v) {
	string *result = new string();
	for (int i = 0; i < (int)v->size(); i++) {
		if (i > 0)
			result->append(" ");
		result->append(getString((*v)[i]));
	}
	return result;
}
