/** @file */ 

#ifndef _DICTIONARY_H_
#define _DICTIONARY_H_

#include <config.h>

#include <string>
#include <vector>
#include <tr1/unordered_map>
#include "common.h"

using namespace std;

/** not present in the dicitonary */
const int DICTIONARY_UNKNOWN = 0;
/** special NULL word (used for "out-of-scope") */
const int DICTIONARY_NULL = 1;
/** minimum id of a word id in the dictionary  */
const int DICTIONARY_MIN_ID = 2;

/**
 * Dictionary implementation. Transform string to integer-id and vice-versa.
 */
class Dictionary {
	tr1::unordered_map<string, int> word2id;
	vector<string*> id2word;
public:
	Dictionary();
	~Dictionary();

	/**
	 * Load a dictionary from the file.
	 * @param file file to read.
	 * @return True if successful.
	 */
	bool load(const char *file);
	/**
	 * Save a dictionary to the file.
	 * @param file file to write.
	 * @return True if successful.
	 */
	bool save(const char *file);
	/**
	 * Return integer-id of a string.
	 * @param str string to transform.
	 * @param insert insert if not found?
	 * @return Integer-id of str.
	 */
	int getId(const char *str, bool insert = false);
	/**
	 * Return string for the integer-id.
	 * @param i integer-id.
	 * @return String binded with i.
	 */
	const char *getString(int i);
	/**
	 * Return string representation of a integer vector. Values are separated by spaces.
	 * @param v vector of integers.
	 * @return String translation of integer-ids.
	 */
	string *decodeVector(vector<int> *v);
};

#endif
