/**
 */

#include <config.h>

#include "feature_functions/list.inc"
#include "FeatureFunctions.h"

FeatureFunctions::FeatureFunctions(Dictionary *dict, WordBuffer *wb) {
	this->dict = dict;
	this->wb = wb;
}

FeatureFunctions::~FeatureFunctions() {
	for (vector<FeatureFunction*>::iterator iter = functions.begin(); iter != functions.end(); ++iter)
		delete (*iter);
}

void FeatureFunctions::init() {
	id2function.clear();
	for (vector<FeatureFunction*>::iterator iter = functions.begin(); iter != functions.end(); ++iter)
		delete (*iter);
	functions.clear();
}

// get id of transformation function, input: normalized string (no spaces)
int FeatureFunctions::convertFunctionId(const char *str) {
	tr1::unordered_map<string, int>::iterator iter = id2function.find(str);
	if (iter == id2function.end()) {
		int id = functions.size();
		id2function[str] = id;

		char *basename = new char[strlen(str)+1];
		strcpy(basename, str);
		char *s = basename;
		while (*s && *s != '(') {
			*s = tolower(*s);
			s++;
		}
		char *args = *s == '(' ? s+1 : NULL;
		*s = '\0';
		// delete trailing ')'
		int len = strlen(args);
		if (len > 0 && args[len-1] == ')')
			args[len-1] = '\0';

		FeatureFunction *ff = getFeatureFunction(dict, wb, basename, args);
		if (!ff) {
			delete[] basename;
			fprintf(stderr, "Unknown function: %s\n", str);
			return FUNCTIONID_NULL;
		}
		delete[] basename;

		// store new function
		functions.push_back(ff);
		return id;
	} else {
		return iter->second;
	}
}
