/**
 * Base class of functions used in features (e.g. substr())
 */
#ifndef _FEATURE_FUNCTIONS_H_
#define _FEATURE_FUNCTIONS_H_

#include <config.h>

#include <assert.h>
#include <tr1/unordered_map>
#include <string>
#include <vector>
#include "Dictionary.h"
#include "WordBuffer.h"
#include "FeatureFunction.h"

const int FUNCTIONID_NULL = -1;

class FeatureFunctions {
	tr1::unordered_map<string, int> id2function;
	vector<FeatureFunction*> functions;
	Dictionary *dict;
	WordBuffer *wb;
public:
	FeatureFunctions(Dictionary *dict, WordBuffer *wb);
	~FeatureFunctions();

	void init();
	int convertFunctionId(const char *str);
	int getFunctionCount();
	const char *getFunctionName(int id);
	FeatureFunction *getFunction(int functionId);
};

inline int FeatureFunctions::getFunctionCount() {
	return functions.size();
}

inline const char *FeatureFunctions::getFunctionName(int id) {
	assert(id < (int)functions.size());
	return functions[id]->getName();
}

inline FeatureFunction *FeatureFunctions::getFunction(int functionId) {
	return functions[functionId];
}

#endif
