/**
 * Feature: one feature
 */
#ifndef _FEATURE_SEGMENT_H_
#define _FEATURE_SEGMENT_H_

#include <config.h>

#include <assert.h>
#include <stdio.h>
#include <limits>
#include <string>
#include "Dictionary.h"
#include "WordBuffer.h"
#include "FeatureFunctions.h"
#include "FeatureFunction.h"
#include "FeatureFunctionRow.h"
#include "FeatureFunctionColumn.h"
#include "FeatureFunctionTransform.h"

using namespace std;

class FeatureSegment {
	Dictionary *dict;
	WordBuffer *wb;
	FeatureFunctions *func;
public:
	int row;
	int col;
	int rowFunctionId;
	int colFunctionId;
	int transFunctionId;
	int value;

	FeatureSegment(Dictionary *dict, WordBuffer *wb, FeatureFunctions *func);
	~FeatureSegment() {};

	void init(int row, int col, int rowFunctionId, int colFunctionId, int transFunctionId);
	void init(int32_t *data);
	char *parse(char *s);
	void update(int sentenceIndex, bool useCorrectTags);
	void getTagContextSize(int *min, int *max);

	string *toString(bool printValue = false);
};

inline void FeatureSegment::update(int sentenceIndex, bool useCorrectTags) {
	int origIndex = sentenceIndex;
	if (rowFunctionId != FUNCTIONID_NULL) {
		int r = static_cast<FeatureFunctionRow*>(func->getFunction(rowFunctionId))->getRow(sentenceIndex, useCorrectTags);
		if (r == std::numeric_limits<int>::min()) {
			value = DICTIONARY_NULL;
			return;
		}
		sentenceIndex = r;
	} else {
		sentenceIndex += row;
	}

	if (colFunctionId != FUNCTIONID_NULL) {
		value = static_cast<FeatureFunctionColumn*>(func->getFunction(colFunctionId))->getValue(origIndex, sentenceIndex, useCorrectTags);
	} else {
		if (col != TAG_COLUMN) {
			value = wb->getFeatureValue(sentenceIndex, col);
		} else {
			assert(row <= 0);
			value = useCorrectTags ? wb->getCorrectTagValue(sentenceIndex) : wb->getProposedTagValue(sentenceIndex);
		}
	}

	if (transFunctionId != FUNCTIONID_NULL) {
		value = static_cast<FeatureFunctionTransform*>(func->getFunction(transFunctionId))->transformValue(value, useCorrectTags);
	}
}

inline void FeatureSegment::getTagContextSize(int *min, int *max) {
	if (rowFunctionId != FUNCTIONID_NULL) {
		static_cast<FeatureFunctionRow*>(func->getFunction(rowFunctionId))->getTagContextSize(min, max);
	} else {
		if (col == TAG_COLUMN) {
			if (row < *min)
				*min = row;
			if (row > *max)
				*max = row;
		}
	}
}

struct feature_segment_hash: public unary_function<FeatureSegment*, size_t> {
	size_t operator() (const FeatureSegment *fs) const {
		return tr1::hash<int>()(((fs->row+31+fs->col*13) + fs->rowFunctionId)*37 + fs->colFunctionId*53 + fs->transFunctionId*17);
	}
};

struct feature_segment_equal {
	bool operator()(const FeatureSegment* fs1, const FeatureSegment* fs2) const {
		if (fs1->rowFunctionId != fs2->rowFunctionId
			|| fs1->colFunctionId != fs2->colFunctionId
			|| fs1->transFunctionId != fs2->transFunctionId
			|| fs1->row != fs2->row || fs1->col != fs2->col)
				return false;
		return true;
	}
};

#endif
