/**
 * Feature: one feature
 */
#ifndef _FEATURE_TEMPLATE_H_
#define _FEATURE_TEMPLATE_H_

#include <config.h>

#include <assert.h>
#include <stdio.h>
#include <tr1/unordered_map>
#include <string>
#include <vector>
#include "Dictionary.h"
#include "WordBuffer.h"
#include "FeatureSegment.h"
#include "FeatureFunction.h"

using namespace std;

// shallow copy (in fact we just copy a pointer)
class FeatureValue {
public:
	int count;
	int *values;

	FeatureValue(int count, int *values): count(count), values(values) {};
	~FeatureValue() {};
};

struct feature_value_hash: public unary_function<FeatureValue&, size_t> {
	size_t operator() (const FeatureValue &fv) const {
		size_t h = 0;
		for (int i = 0; i < fv.count; i++)
			h = h * 37 + fv.values[i];
		return tr1::hash<int>()(h);
	}
};

struct feature_value_equal {
	bool operator()(const FeatureValue& fv1, const FeatureValue& fv2) const {
		for (int i = 0; i < fv1.count; i++) {
			if (fv1.values[i] != fv2.values[i])
				return false;
		}
		return true;
	}
};

struct FeatureTemplate {
	// segments for the current feature teplate
	int segmentCount;
	int *segmentIndices;
	// features (template instances), real data are kept in featureData (for efficiency)
	tr1::unordered_map<FeatureValue, int, feature_value_hash, feature_value_equal> featureIndex;
	int *featureData;
	// feature counts (for expanding feature templates into features)
	tr1::unordered_map<FeatureValue, int, feature_value_hash, feature_value_equal> featureCount;

	// used by getCurrentFeatureIndex()
	int *tmpValues;

	Dictionary *dict;
	WordBuffer *wb;
	FeatureFunctions *func;
	vector<FeatureSegment*> *segments;
	tr1::unordered_map<FeatureSegment*, int, feature_segment_hash, feature_segment_equal> *segmentIndex;
public:
	FeatureTemplate(Dictionary *dict, WordBuffer *wb, FeatureFunctions *func, vector<FeatureSegment*> *segments, tr1::unordered_map<FeatureSegment*, int, feature_segment_hash, feature_segment_equal> *segmentIndex);
	~FeatureTemplate();

	void init(int count, int32_t *data);
	int getSegmentCount();
	int *getSegmentIndices();
	bool parseFeatureTemplate(char *s);
	void expandFeatureTemplate();
	int createFeatures(int minFreq, int totalFeatures);
	int addFeatures(int count, int32_t *data, int totalFeatures);
	void getFeatures(int *count, int **data);
	int getFeatureIndex(FeatureValue *value);
	int getCurrentFeatureIndex();
	void getCurrentValue(FeatureValue *fv);

	string *toString(bool currentOnly = false);
};

inline int FeatureTemplate::getSegmentCount() {
	return segmentCount;
}

inline int *FeatureTemplate::getSegmentIndices() {
	return segmentIndices;
}

// returns -1 if feature is not found
inline int FeatureTemplate::getFeatureIndex(FeatureValue *value) {
	tr1::unordered_map<FeatureValue, int, feature_value_hash, feature_value_equal>::iterator iter = featureIndex.find(*value);
	return iter != featureIndex.end() ? iter->second : -1;
}

// returns -1 if feature is not found
inline int FeatureTemplate::getCurrentFeatureIndex() {
	for (int i = 0; i < segmentCount; i++)
		tmpValues[i] = (*segments)[segmentIndices[i]]->value;
	FeatureValue fv(segmentCount, tmpValues);
	return getFeatureIndex(&fv);
}

// ResultValue must have enough space available
inline void FeatureTemplate::getCurrentValue(FeatureValue *fv) {
	assert(fv->count >= segmentCount);
	for (int i = 0; i < segmentCount; i++)
		fv->values[i] = (*segments)[segmentIndices[i]]->value;
}

#endif
