/** @file */
#ifndef _FEATURES_H_
#define _FEATURES_H_

#include <config.h>

#include <stdint.h>
#include <stdio.h>
#include <tr1/unordered_map>
#include <tr1/unordered_set>
#include <string>
#include <vector>
#include "common.h"
#include "FeatureTemplate.h"
#include "Dictionary.h"
#include "WordBuffer.h"

using namespace std;

#if ALPHA_SIZE == 64
	typedef int64_t alpha_t;
#else
	typedef int32_t alpha_t;
#endif

class Features {
	Dictionary *dict;
	WordBuffer *wb;
	FeatureFunctions *func;
	vector<FeatureTemplate*> featureTemplates;
	int totalFeatures;
	vector<FeatureSegment*> segments;
	tr1::unordered_map<FeatureSegment*, int, feature_segment_hash, feature_segment_equal> segmentIndex;

	alpha_t *alphas;
	alpha_t *gammas;
	// for speedup of gamma update
	int *lastGammaUpdate;
	int sentenceId;
	unsigned sentenceCount;

	// used by countScore
	int *prevContextSize;
	int *nextContextSize;

	// used by UpdateAlphas()
	int featureTemplateCount;
	int *decAlphaIndex;
	int *incAlphaIndex;

	bool _loadFeatureTemplates(FILE *f);
	bool _loadFeatures(FILE *f);
	bool _saveFeatures(FILE *f, int minFreq);
public:
	//Features(Dictionary *dict, WordBuffer *wb, bool useGamma = false);
	Features(Dictionary *dict, WordBuffer *wb);
	~Features();

	void init();

	bool loadFeatureTemplates(const char *file);
	bool loadFeatures(const char *file);
	bool saveExpandedFeatures(const char *file, int minFreq = 1);

	void initAlphas();
	void finishAlphas();
	bool loadAlphas(const char *file);
	bool saveAlphas(const char *file);

	void updateAlphas();
	void expandFeatureTemplates(int sentenceIndex);
	double countScore(int sentenceIndex, bool useGamma);
	double countScore(int sentenceIndex, int prevContextSize, int nextContextSize, bool useGamma);
	int getSentenceCount();
	void getTagContextSize(int *min, int *max);
	void updateSegments(int sentenceIndex, bool useCorrectTags);

	string *toString();
};

inline int Features::getSentenceCount() {
	return sentenceCount;
}

#endif
