/**
 * GuidedLearning
 */

#include <config.h>

#include <stdio.h>
#include "GuidedLearning.h"

GuidedLearning::GuidedLearning(Dictionary *dict, WordBuffer *wb, int beamWidth, int interfaceSize) {
	this->dict = dict;
	this->wb = wb;

	Span::Init(wb, &spans, beamWidth, interfaceSize);
}

GuidedLearning::~GuidedLearning() {
	Span::Destroy();
}

void GuidedLearning::tagSentence(Features *ftrs) {
	// create a span for every word in a sentence
	int sentenceSize = wb->getSentenceSize();
	Span::initSentence(sentenceSize);

	// count hypothesis/action scores for all untagged (= all) candidates
	for (int i = 0; i < sentenceSize; i++)
		spans[i]->updateScore(ftrs);

	// main tagging cycle
	for (int i = 0; i < sentenceSize; i++) {
		// find top action score
		int bestIndex = Span::getTopActionScoreIndex(i == 0);
		// tag the span
		spans[bestIndex]->tagSpan();
		// update preceeding/following spans
		Span *prev = spans[bestIndex]->getPreviousSpan();
		if (prev)
			prev->updateScore(ftrs);
		Span *next = spans[bestIndex]->getFollowingSpan();
		if (next)
			next->updateScore(ftrs);
	}
}

void GuidedLearning::trainSentence(Features *ftrs) {
	// create a span for every word in a sentence
	int sentenceSize = wb->getSentenceSize();
	Span::initSentence(sentenceSize);

	// count hypothesis/action scores for all untagged (= all) candidates
	for (int i = 0; i < sentenceSize; i++)
		spans[i]->updateScore(ftrs);

	// main tagging cycle
	for (int i = 0; i < sentenceSize; i++) {
		// find top action score
		int bestIndex = Span::getTopActionScoreIndex(i == 0);
		// tag the span
		spans[bestIndex]->tagSpan();
		// update preceeding/following spans
		Span *prev = spans[bestIndex]->getPreviousSpan();
		if (prev)
			prev->updateScore(ftrs);
		Span *next = spans[bestIndex]->getFollowingSpan();
		if (next)
			next->updateScore(ftrs);
	}
}

string *GuidedLearning::toString() {
	//TODO
	return new string("");
}
