/**
 * Span
 */

#include <config.h>

#include <stdio.h>
#include <algorithm>
#include "Span.h"

int Span::beamWidth;
int Span::interfaceSize;
vector<Span*> *Span::spans;
tr1::unordered_map<int, list<Span::UsedFeature> > *Span::usedFeatures;
Heap<Span::ActionScore> Span::actionScoreHeap;
WordBuffer *Span::wb;

// State

void Span::State::set(double topHypothesisScore, int tagIndex) {
	this->topHypothesisScore = topHypothesisScore;
	this->tagIndex = tagIndex;
}

string *Span::State::toString() {
	//TODO
	return new string("");
}

// HypothesisInterface
void Span::HypothesisInterface::set(Span *span, int hypothesisIndex) {
	// TODO
}

// Span

Span::Span(int sentenceIndex) {
//FIXME: check all inits
	states = new State[beamWidth];
	hypothesisCount = 0;
	bestActionScore = NULL;
	reset(sentenceIndex);
}

Span::~Span() {
	for (int i = 0; i < hypothesisCount; i++)
		delete hypotheses[i];
	hypotheses.clear();
	delete[] states;
}

void Span::reset(int sentenceIndex) {
	tagged = false;
	firstWordIndex = lastWordIndex = sentenceIndex;
	stateCount = 0;
	hypothesisCount = 0;
}

void Span::Init(WordBuffer *wb, vector<Span*> *spans, int beamWidth, int interfaceSize) {
	Span::wb = wb;
	Span::beamWidth = beamWidth;
	Span::interfaceSize = interfaceSize;
}

void Span::Destroy() {
}


bool Span::equivalentStates(int leftStateIndex1, int leftStateIndex2, int rightStateIndex1, int rightStateIndex2) {
	Span *prev = getPreviousSpan();
	if (prev && prev->isTagged()) {
		int prevLength = prev->getSpanLength();
		if (prevLength < interfaceSize)
			return false;
		int first = prev->getFirstWordIndex();
		for (int i = 0; i < interfaceSize; i++) {
			if (!(*spans)[first+i]->compatibleStateTag(leftStateIndex1, leftStateIndex2))
				return false;
		}
	}

	Span *next = getFollowingSpan();
	if (next && next->isTagged()) {
		int nextLength = next->getSpanLength();
		if (nextLength < interfaceSize)
			return false;
		int last = next->getLastWordIndex();
		for (int i = 0; i < interfaceSize; i++) {
			if (!(*spans)[last-i]->compatibleStateTag(rightStateIndex1, rightStateIndex2))
				return false;
		}
	}
	return true;
}

// return interface size
int Span::fillTags(int sentenceIndex, int stateIndex, bool leftInterface) {
	int len = getSpanLength();
	int i;
	if (leftInterface) {
		for (i = 0; i < interfaceSize && i < len; i++) {
			int tagIndex = (*spans)[firstWordIndex+i]->getState(stateIndex)->getTagIndex();
			wb->setProposedTagIndex(sentenceIndex, tagIndex);
			sentenceIndex++;
		}
	} else {
		for (i = 0; i < interfaceSize && i < len; i++) {
			int tagIndex = (*spans)[lastWordIndex-i]->getState(stateIndex)->getTagIndex();
			wb->setProposedTagIndex(sentenceIndex, tagIndex);
			sentenceIndex--;
		}
	}
	return i;
}

void Span::updateScore(Features *ftrs) {
	assert(!isTagged());

	int sentenceIndex = firstWordIndex;
	hypothesisCount = 0;

	int prevStateCount = 0;
	int nextStateCount = 0;

	Span *prev = getPreviousSpan();
	if (prev) {
		if (prev->isTagged())
			prevStateCount = prev->getStateCount();
		else
			prev = NULL;
	}
	Span *next = getFollowingSpan();
	if (next) {
		if (next->isTagged())
			nextStateCount = next->getStateCount();
		else
			next = NULL;
	}

	double topHypothesisScore = std::numeric_limits<double>::min();
	double topActionScore = std::numeric_limits<double>::min();

	// for all previous states
	for (int i = 0; i < prevStateCount || (!prev && i == 0); i++) {
		int prevContextSize = 0;
		double prevHypoScore = 0;
		if (prev) {
			prevContextSize = prev->fillTags(sentenceIndex, i, false);
			prevHypoScore = prev->getState(i)->getTopHypothesisScore();
		}
		// for all next states
		for (int j = 0; j < nextStateCount || (!next && j == 0); j++) {
			int nextContextSize = 0;
			double nextHypoScore = 0;
			if (next) {
				nextContextSize = next->fillTags(sentenceIndex, j, true);
				nextHypoScore = next->getState(i)->getTopHypothesisScore();
			}

			// for all tags in the offer
			int offerSize = wb->getOfferSize(sentenceIndex);
			for (int k = 0; k < offerSize; k++) {
				// only consider compatible features
				double actionScore = ftrs->countScore(sentenceIndex, prevContextSize, nextContextSize, true/*FIXME*/);
				double score = prevHypoScore + actionScore + nextHypoScore;

				// compare to previous (to save some space?) -- TODO: review if this is needed (ma to nejaky vyznam)
				// FIXME: cannot do this in training
				if (hypothesisCount > 0) {
					Hypothesis *hypo = hypotheses[hypothesisCount-1];
					if (equivalentStates(hypo->getLeftStateIndex(), i, hypo->getRightStateIndex(), j))
						continue;
				}

				// save hypothesis
				if ((unsigned)hypothesisCount == hypotheses.size())
					hypotheses.push_back(new Hypothesis());
				hypotheses[hypothesisCount]->set(score, actionScore, k, i, j);

				// save action score of the top hypothesis
				if (score > topHypothesisScore) {
					topHypothesisScore = score;
					topActionScore = actionScore;
				}
			}
		}
	}
	// create or update item of action score heap for this sentence index
	if (bestActionScore) {
		actionScoreHeap.remove(bestActionScore->getHeapIndex());
		actionScoreHeap.getFirstUnused()->set(sentenceIndex, topActionScore);
		actionScoreHeap.insertFirstUnused();
	} else {
		actionScoreHeap.getFirstUnused()->set(sentenceIndex, topActionScore);
		actionScoreHeap.insertFirstUnusedRaw();
	}
}

/////////////
#if 0
				// save hypothesis
				if (hypothesisCount < beamWidth) {
					// insert new hypothesis
					hypotheses[hypothesesCount]->set(score, actionScore, i, j);
					if (hypothesesCount == 0 || score < lowestHypothesisScore)
						lowestHypothesesScore = score;
					if (++hypothesisCount == beamWidth)
						make_heap(hypotheses, hypotheses + beamWidth);
				} else if (score > lowestHypothesisScore) {
					// update top-hypothesis score heap
					pop_heap(hypotheses, hypotheses + beamWidth);
					hypotheses[beamWidth-1]->set(score, actionScore, i, j);
					pop_heap(hypotheses, hypotheses + beamWidth);
				}
/////////////
			}
		}
	}
}
#endif

void Span::tagSpan() {
	// craete heap from hypotheses
	make_heap(hypotheses.begin(), hypotheses.begin()+hypothesisCount);

	// find N top-scoring hypotheses with different states
	tr1::unordered_set<HypothesisInterface*, hypothesis_interface_hash, hypothesis_interface_equal> *checked = new tr1::unordered_set<HypothesisInterface*, hypothesis_interface_hash, hypothesis_interface_equal>();

	int *selected = new int[beamWidth];
	int selectedCount = 0;
	HypothesisInterface **his = new HypothesisInterface*[beamWidth];
	HypothesisInterface *hi = new HypothesisInterface();
	while (hypothesisCount > 0 && selectedCount < beamWidth) {
		pop_heap(hypotheses.begin(), hypotheses.begin() + hypothesisCount);
		hi->set(this, hypothesisCount-1);
		hypothesisCount--;
		if (selected > 0 && checked->find(hi) != checked->end())
			continue;
		checked->insert(hi);
		his[selectedCount] = hi;
		selected[selectedCount] = hypothesisCount;
		selectedCount++;
	}
	delete checked;

	// create new states from top hypotheses
	for (int i = 0; i < selectedCount; i++) {
		// FIXME TODO: nejak chytre zkopirovat tagy ze stavu do stavu
		// zkopirovat/upravit firstWordIndex/lastWordIndex
		delete his[i];
	}
	delete[] his;
	delete[] selected;

	// remove head of the heap (we just tagged it :-))
	actionScoreHeap.remove(0);
}

string *Span::toString() {
	//TODO
	return new string("");
}

// adjust sentence-wide (static) variables
void Span::initSentence(int sentenceSize) {
	int size = spans->size();

	// spans array
	for (int i = 0; i < sentenceSize; i++) {
		if (i < size)
			(*spans)[i]->reset(i);
		else
			spans->push_back(new Span(i));
	}

	// actionScore heap
	actionScoreHeap.clear();

	// reset usedFeatures hash (+lists)
	if (usedFeatures)
		usedFeatures->clear();
}

int Span::getTopActionScoreIndex(bool buildHash) {
	// create the heap (in case we have full index
	if (buildHash)
		actionScoreHeap.rebuild();
	// return head of the heap
	return actionScoreHeap.get(0)->getSentenceIndex();
}
