/*
 * Single word in a WordBuffer.
 */

#include <config.h>

#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "Word.h"

Word::Word(int tagIndex, int maxProposed, int maxOffer) {
	features = new int[tagIndex];
	proposedTagIndex = new int[maxProposed];
	proposedSize = 0;
	offer = new int[maxOffer];
	offerSize = 0;
	additionalOffer = NULL;
}

Word::~Word() {
	delete[] features;
	delete[] proposedTagIndex;
	delete[] offer;
	delete[] additionalOffer;
}

bool Word::parse(Dictionary *dict, bool appendDict, bool addCorrectTag, int tagIndex, int maxOffer, const char *str) {
	char *s = strdup(str);
	char *helper = NULL;
	char *token = f_strtok_r(s, "\t", &helper);
	int i = 0;
	bool tagSeenInOffer = false;
	while (token) {
		if (i < tagIndex) {
			features[i] = dict->getId(token, appendDict);
		} else if (i == tagIndex) {
			char *sp = strchr(token, ' ');
			if (sp)
				*sp = '\0';
			correctTag = dict->getId(token, appendDict);
			// save additional info for the correct tag
			if (sp)
				additionalCorrectTag = dict->getId(sp+1, appendDict);
		} else {
			int index = i-(tagIndex+1);
			assert(index < maxOffer);
			// additional info for a tag
			char *sp = strchr(token, ' ');
			if (sp)
				*sp = '\0';
			int strid = dict->getId(token, appendDict);
			offer[index] = strid;
			// save additional info for a tag
			if (sp) {
				if (!additionalOffer)
					additionalOffer = new int[maxOffer];
				additionalOffer[index] = dict->getId(sp+1, appendDict);
			} else if (additionalOffer) {
				additionalOffer[index] = DICTIONARY_NULL;
			}
			if (strid == correctTag) {
				if (!additionalOffer)
					tagSeenInOffer = true;
				else if (additionalOffer[index] == additionalCorrectTag)
					tagSeenInOffer = true;
			}
		}
		token = f_strtok_r(NULL, "\t", &helper);
		i++;
	}
	free(s);

	// not enough fields
	if (i <= tagIndex) {
		errno = 0;
		return false;
	}

	// no target tag in offer
	if (addCorrectTag && !tagSeenInOffer) {
		int index = i-(tagIndex+1);
		assert(index < maxOffer);
		offer[index] = correctTag;
		if (additionalOffer)
			additionalOffer[index] = additionalCorrectTag;
		i++;
	}
	offerSize = i - (tagIndex+1);
	errno = 0;
	return offerSize > 0 ? true : false;
}

string *Word::toString(Dictionary *dict, int tagIndex) {
	string *result = new string();

	for (int i = 0; i < tagIndex; i++) {
		result->append(dict->getString(features[i]));
		result->append(" ");
	}

	result->append("[");
	result->append(dict->getString(correctTag));
	if (additionalOffer) {
		result->append(" (");
		result->append(dict->getString(additionalCorrectTag));
		result->append(")");
	}
	result->append(", ");
	for (int i = 0; i < proposedSize; i++) {
		result->append(dict->getString(getProposedTagValue(i)));
		if (additionalOffer) {
			result->append(" (");
			result->append(dict->getString(getAdditionalProposedTagValue(i)));
			result->append(")");
		}
		result->append(" ");
	}
	result->append("] [");

	for (int i = 0; i < offerSize; i++) {
		if (i > 0)
			result->append(" ");
		result->append(dict->getString(offer[i]));
		if (additionalOffer) {
			result->append(" (");
			result->append(dict->getString(additionalOffer[i]));
			result->append(")");
		}
	}
	result->append("]");

	return result;
}
