/** @file */

/* Flat C-style interface to Collins' perceptron algorithm implementation. */

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>

/** opaque structure, used to keep the current state. It should be initialized
 * using perc_init() and destroyed using perc_destro() */
typedef struct perc PERC;

/**
 * Initialize PERC structure.
 * @return Initialized structure
 */
PERC *perc_init();
/**
 * Free all objects and data structures associated with current PERC structure
 * @param perc PERC structure
 */
void perc_destroy(PERC *perc);

/**
 * Free all objects and data structures associated with current PERC structure
 * @param perc PERC structure
 */
int perc_process_file_full(PERC *perc, const char *dataFile, int skipHeader);
int perc_process_file(PERC *perc, FILE *dataFile);

int perc_append_word(PERC *perc, const char *line);
void perc_begin_sentence(PERC *perc);
void perc_end_sentence(PERC *perc);

void perc_get_accuracy(PERC *perc, double *correct, double *additionalCorrect);
void perc_reset_accuracy(PERC *perc);
const char *perc_get_proposed_tag(PERC *perc, int sentenceIndex, int index);
const char *perc_get_additional_proposed_tag(PERC *perc, int sentenceIndex, int index);
double perc_get_nbest_score(PERC *perc, int index);

/**
 * Initialize procedure of crating the feature file (preceeds the training step).
 * @param templateFile feature templates file (text file)
 * @param dictFile input dictionary file, usually not needed
 * @param header header description of the data file (column names are features@see ... for the data file format description)
 * @return zero in case of error
 */
int perc_make_features_init(PERC *perc, const char *templateFile, const char *dictFile, const char *header);
int perc_make_features_finish(PERC *perc, const char *featureFile, const char *dictFile, int minFreq);

int perc_train_init(PERC *perc, const char *featureFile, const char *dictFile, const char *alphaFile, const char *header, int prune, int order);
int perc_train_finish(PERC *perc, const char *dictFile, const char *alphaFile);

int perc_test_init(PERC *perc, const char *featureFile, const char *dictFile, const char *alphaFile, const char *header, int prune, int nbest, int order);
int perc_test_finish(PERC *perc);

#ifdef __cplusplus
}
#endif

