package Featurama::Perc;

use strict;
use Featurama;
use Exporter;
use Encode;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION     = '1.00';
@ISA         = qw(Exporter);
@EXPORT      = ();
@EXPORT_OK   = qw(new processFile appendWord beginSentence endSentence
		getAccuracy getProposedTag getAdditionalProposedTag getNBestScore
		makeFeaturesInit makeFeaturesFinish
		trainInit trainFinish
		testInit testFinish);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = {};
	$self->{PERC} = &Featurama::perc_init();
	bless($self, $class);
	return $self;
}

sub DESTROY {
	my $self = shift;
	# FIXME: $self->{PERC} should be always defined (but it is not in TMT)
	&Featurama::perc_destroy($self->{PERC}) if (defined $self->{PERC});
}

sub processFile() {
	my ($self, $dataFile, $skipHeader) = @_;
	return &Featurama::perc_process_file_full($self->{PERC}, $dataFile, $skipHeader);
}

sub appendWord() {
	my ($self, $line) = @_;
	return &Featurama::perc_append_word($self->{PERC}, $line);
}

sub beginSentence() {
	my ($self) = @_;
	&Featurama::perc_begin_sentence($self->{PERC});
}

sub endSentence() {
	my ($self) = @_;
	&Featurama::perc_end_sentence($self->{PERC});
}

sub getAccuracy() {
	my ($self) = @_;
	my ($correct, $additional) = &Featurama::perc_get_accuracy($self->{PERC});
	return ($correct, $additional);
}

sub resetAccuracy() {
	my ($self) = @_;
	&Featurama::perc_reset_accuracy($self->{PERC});
}

sub getProposedTag() {
	my ($self, $sentenceIndex, $index) = @_;
	my $tag = &Featurama::perc_get_proposed_tag($self->{PERC}, $sentenceIndex, $index);
	Encode::_utf8_on($tag);
	return $tag;
}

sub getAdditionalProposedTag() {
	my ($self, $sentenceIndex, $index) = @_;
	my $tag = &Featurama::perc_get_additional_proposed_tag($self->{PERC}, $sentenceIndex, $index);
	Encode::_utf8_on($tag);
	return $tag;
}

sub getNBestScore() {
	my ($self, $index) = @_;
	return &Featurama::perc_get_nbest_score($self->{PERC}, $index);
}

sub makeFeaturesInit() {
	my ($self, $templateFile, $dictFile, $header) = @_;
	return &Featurama::perc_make_features_init($self->{PERC}, $templateFile, $dictFile, $header);
}

sub makeFeaturesFinish() {
	my ($self, $featureFile, $dictFile, $minFreq) = @_;
	return &Featurama::perc_make_features_finish($self->{PERC}, $featureFile, $dictFile, $minFreq);
}

sub trainInit() {
	my ($self, $featureFile, $dictFile, $alphaFile, $header, $prune, $order) = @_;
	return &Featurama::perc_train_init($self->{PERC}, $featureFile, $dictFile, $alphaFile, $header, $prune, $order);
}

sub trainFinish() {
	my ($self, $dictFile, $alphaFile) = @_;
	return &Featurama::perc_train_finish($self->{PERC}, $dictFile, $alphaFile);
}

sub testInit() {
	my ($self, $featureFile, $dictFile, $alphaFile, $header, $prune, $nbest, $order) = @_;
	return &Featurama::perc_test_init($self->{PERC}, $featureFile, $dictFile, $alphaFile, $header, $prune, $nbest, $order);
}

sub testFinish() {
	my ($self) = @_;
	return &Featurama::perc_test_finish($self->{PERC});
}

1;
