#!/usr/bin/env bash

base=`dirname $0`
cd "$base"

if [ "$1" = "clean" ]; then
	rm -f allsamples.zip FMorph.tgz mdata2csts.xsl ../train_pdt.data ../test_pdt.data
	exit 0
fi

test ! -f ../train_pdt.data -o ! -f ../test_pdt.data || exit 0

function test_bin() {
	while [ $# -gt 0 ]; do
		if ! which $1 >/dev/null; then
			echo $1 not found.
			exit 1
		fi
		shift
	done
}

test_bin wget unzip tar xsltproc

# get Czech Morphology (Free)
if [ ! -f FMorph.tgz ]; then
	wget http://ufal.mff.cuni.cz/pdt/Morphology_and_Tagging/Morphology/Morphological_analyzer/FMorph.tgz || exit 1
fi

# get PDT 2.0 sample data
if [ ! -f allsamples.zip ]; then
	wget http://ufal.mff.cuni.cz/pdt2.0/data/sample/allsamples.zip || exit 1
fi

# get PDT 2.0 conversion script
if [ ! -f mdata2csts.xsl ]; then
	wget http://ufal.mff.cuni.cz/pdt2.0/tools/format-conversions/pdt_formats/mdata2csts.xsl || exit 1
fi

mkdir FMorph 2>/dev/null
tar xzf FMorph.tgz -C FMorph
unzip -q -o allsamples.zip
for f in *.m.gz; do
	b=`basename $f .m.gz`
	echo $b
	xsltproc mdata2csts.xsl $f | sed -e 's/\(<[sdf]\) id=[^>]*>/\1>/' | FMorph/FMAnalyze.pl FMorph/CZE-a.il2 | iconv -f latin2 -t utf8 >$b.data
done
rm -rf sample*.gz FMorph

# any UTF-8 locale should do
cat sample[0-7].data | LC_ALL=en_US.utf8 ../../utils/csts/csts2perc-cs.pl train >../train_pdt.data
cat sample{8,9}.data | LC_ALL=en_US.utf8 ../../utils/csts/csts2perc-cs.pl test >../test_pdt.data
rm -f sample[0-9].data
