#!/usr/bin/env bash

base=`dirname $0`
cd "$base"

MORPHIUM=morphium080920.tgz

if [ "$1" = "clean" ]; then
	rm -f ../train_wsj.data ../test_wsj.data morphium $MORPHIUM
	exit 0
fi

test ! -f ../train_wsj.data -o ! -f ../test_wsj.data || exit 0

dir=$1
if [ -z "$dir" ]; then
	echo "WSJ data not specified."
	exit 77
fi
if [ ! -f "$dir/00/wsj_0001.mrg" ]; then
	echo "WSJ data not found ($dir/00/wsj_0001.mrg)."
	exit 77
fi

function test_bin() {
	while [ $# -gt 0 ]; do
		if ! which $1 >/dev/null; then
			echo $1 not found.
			exit 1
		fi
		shift
	done
}

test_bin wget tar perl

# get Morphium
if [ ! -f $MORPHIUM ]; then
	wget http://ufal.mff.cuni.cz/morphium/$MORPHIUM || exit 1
fi

rm -rf morphium
tar xzf $MORPHIUM

# create train
( cd "$dir"; for d in 00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18; do
	cat $d/*.mrg
done ) | ./parsed2pos.pl | ( cd morphium; ./run.sh ) | ../../utils/csts/vert2csts.pl | ../../utils/csts/csts2perc-en.pl >../train_wsj.data

# create dtest
( cd "$dir"; for d in 19 20 21; do
	cat $d/*.mrg
done ) | ./parsed2pos.pl | ( cd morphium; ./run.sh ) | ../../utils/csts/vert2csts.pl | ../../utils/csts/csts2perc-en.pl >../test_wsj.data

rm -rf morphium
