#!/usr/bin/env bash
#
# This script tests training + tagging using PDT sample data

base=`dirname $0`
cd "$base"

if [ "$1" = "clean" ]; then
	rm -f test-pdt/delme.*
	exit 0;
fi

# create data
if [ ! -f train_pdt.data -o ! -f test_pdt.data ]; then
	if ! data_pdt/create.sh; then
		echo "Cannot create test data"
		exit 77
	fi
fi

# training results
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/train.sh -v 20 czech-test.ft train_pdt.data test_pdt.data test-pdt/delme >test-pdt/delme.train.out 2>&1
if ! diff -u test-pdt/train.out test-pdt/delme.train.out; then
	echo "Training output differs"
	exit 1
fi
if [ `wc -l test-pdt/delme.dict|cut -f1 -d" "` != `cat test-pdt/dict.wc|cut -f1 -d" "` ]; then
	echo "Dictionary file size differs"
	exit 1
fi

# simple test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh test-pdt/delme test_pdt.data >test-pdt/delme.test.out 2>&1
if ! diff -u test-pdt/test.out test-pdt/delme.test.out; then
	echo "Testing output differs"
	exit 1
fi

# n-best test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh -b 10 -s test-pdt/delme test_pdt.data >test-pdt/delme.nbtest.out 2>&1
if ! diff -u test-pdt/nbtest.out test-pdt/delme.nbtest.out; then
	echo "N-best testing output differs"
	exit 1
fi

# prunning
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/train.sh -v -p 5 20 czech-test.ft train_pdt.data test_pdt.data test-pdt/delme.p >test-pdt/delme.ptrain.out 2>&1
if ! diff -u test-pdt/ptrain.out test-pdt/delme.ptrain.out; then
	echo "Prunning trainoutput differs"
	exit 1
fi
if [ `wc -l test-pdt/delme.dict|cut -f1 -d" "` != `cat test-pdt/dict.wc|cut -f1 -d" "` ]; then
	echo "Dictionary file size differs"
	exit 1
fi

# pruning test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh test-pdt/delme.p test_pdt.data >test-pdt/delme.ptest.out 2>&1
if ! diff -u test-pdt/ptest.out test-pdt/delme.ptest.out; then
	echo "Prunning test output differs"
	exit 1
fi

# pruning + n-best test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh -b 10 -s test-pdt/delme.p test_pdt.data >test-pdt/delme.pnbtest.out 2>&1
if ! diff -u test-pdt/pnbtest.out test-pdt/delme.pnbtest.out; then
	echo "N-best + prunning test output differs"
	exit 1
fi

rm -f test-pdt/delme.*

exit 0
