#!/usr/bin/env bash
#
# This script tests training + tagging using WSJ parsed data (*.mrg)

base=`dirname $0`
cd "$base"

if [ "$1" = "clean" ]; then
	rm -f test-wsj/delme.*
	exit 0;
fi

# create data
if [ ! -f train_wsj.data -o ! -f test_wsj.data ]; then
	if ! data_wsj/create.sh wsj; then
		echo "Cannot create test data"
		exit 77
	fi
fi

# training results
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/train.sh -v 2 english-test.ft train_wsj.data test_wsj.data test-wsj/delme >test-wsj/delme.train.out 2>&1
if ! diff -u test-wsj/train.out test-wsj/delme.train.out; then
	echo "Training output differs"
	exit 1
fi
if [ `wc -l test-wsj/delme.dict|cut -f1 -d" "` != `cat test-wsj/dict.wc|cut -f1 -d" "` ]; then
	echo "Dictionary file size differs"
	exit 1
fi

# simple test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh test-wsj/delme test_wsj.data >test-wsj/delme.test.out 2>&1
if ! diff -u test-wsj/test.out test-wsj/delme.test.out; then
	echo "Testing output differs"
	exit 1
fi

# n-best test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh -b 10 -s test-wsj/delme test_wsj.data >test-wsj/delme.nbtest.out 2>&1
if ! diff -u test-wsj/nbtest.out test-wsj/delme.nbtest.out; then
	echo "N-best testing output differs"
	exit 1
fi

# prunning
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/train.sh -v -p 5 2 english-test.ft train_wsj.data test_wsj.data test-wsj/delme.p >test-wsj/delme.ptrain.out 2>&1
if ! diff -u test-wsj/ptrain.out test-wsj/delme.ptrain.out; then
	echo "Prunning trainoutput differs"
	exit 1
fi
if [ `wc -l test-wsj/delme.dict|cut -f1 -d" "` != `cat test-wsj/dict.wc|cut -f1 -d" "` ]; then
	echo "Dictionary file size differs"
	exit 1
fi

# pruning test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh test-wsj/delme.p test_wsj.data >test-wsj/delme.ptest.out 2>&1
if ! diff -u test-wsj/ptest.out test-wsj/delme.ptest.out; then
	echo "Prunning test output differs"
	exit 1
fi

# pruning + n-best test result
PATH=$PATH:$srcdir/../src TSFX=_x $srcdir/../utils/test.sh -b 10 -s test-wsj/delme.p test_wsj.data >test-wsj/delme.pnbtest.out 2>&1
if ! diff -u test-wsj/pnbtest.out test-wsj/delme.pnbtest.out; then
	echo "N-best + prunning test output differs"
	exit 1
fi

rm -f test-wsj/delme.*

exit 0
