#!/usr/bin/env perl

use strict;
use Data::Dumper;

BEGIN {
	binmode STDIN, ":utf8";
	binmode STDOUT, ":utf8";
	binmode STDERR, ":utf8";
}

# covert Jan Raab's Morce feature file into perc feaeture file.
# usage: morcef2percf.pl morce_dict perc_dict <morce.f >perc.f

(@ARGV == 2) or die("usage: morcef2percf.pl morce_dict perc_dict <morce.f >perc.f");

# read morce_dict
my @morce_dict;
my $fin;
open($fin, "<".$ARGV[0]) or die("Cannot open file ".$ARGV[0]);
my $i = 1;
my $skip = 3;
while (<$fin>) {
	chomp;
	if ($skip > 0) {
		$skip--;
		next;
	}
	$morce_dict[$i] = $_;
	$i++;
}
close($fin);

# read perc_dict
my %perc_dict;
open($fin, "<".$ARGV[1]) or die("Cannot open file ".$ARGV[1]);
$i = 3;
while (<$fin>) {
	chomp;
	$perc_dict{$_} = $i;
	$i++;
}
close($fin);

sub number {
	my ($l) = @_;
	my $i = 0;
	my $sign = 1;
	my $c = shift(@{$l});
	if ($c eq "-") {
		$sign = -1;
		$c = shift(@{$l});
	}
	while (defined($c)) {
		if ($c ge "0" and $c le "9") {
			$i = $i * 10 + ord($c)-ord("0");
		} else {
			unshift(@{$l}, $c);
			last;
		}
		$c = shift(@{$l});
	}
	return $i * $sign;
}

sub translate {
	my ($l) = @_;
	my $c = shift(@{$l});
	return "" if (not defined $c);
	unshift(@{$l}, $c);
	my $i = &number(\@{$l});
	if ($i == -1) {
		return "NULL";
	} else {
		return defined $morce_dict[$i] ? $morce_dict[$i] : "???";
	}
}

while (<STDIN>) {
	chomp;
	s/\r$//;
	my $first = 1;
	next if (/^#/);
#print STDERR $_."\n";
	my @l = split(//);
	while (scalar(@l) > 0) {
		# position
		my $c = shift(@l);
		if ($first) {
			$first = 0;
		} else {
			print "/";
		}
		my $pos = "";
		if ($c eq "|") {
			$pos = "0";
		} elsif ($c eq "V") {
			$pos = "V";
		} elsif ($c eq "U") {
			$pos = "U";
		} elsif (ord($c) >= ord("A") and ord($c) <= ord("C")) {
			$pos = -1-ord($c)+ord("A");
		} elsif (ord($c) >= ord("a") and ord($c) <= ord("c")) {
			$pos = 1+ord($c)-ord("a");
		} else {
			die("Unknown position: $c ($_)");
		}
		# what
		$c = shift(@l);
		my @what = ();
		my @word = ();
		my @func = ();
		if ($c eq "T") {
			if ($pos eq "V") {
				$pos = "0";
				push(@what, "Tag");
				push(@word, &translate(\@l));
				push(@func, "PreviousVerb()");
			} elsif ($pos eq "U") {
				$pos = "0";
				push(@what, "FollowingVerbTag");
				push(@word, &translate(\@l));
				push(@func, "");
			} else {
				push(@what, "Tag");
				push(@word, &translate(\@l));
				push(@func, "");
			}
		} elsif ($c eq "W") {
			push(@what, "Form");
			push(@word, &translate(\@l));
			push(@func, "");
		} elsif ($c eq "L") {
			if ($pos eq "V") {
				$pos = "0";
				push(@what, "Tag");
				push(@word, &translate(\@l));
				push(@func, "PreviousVerbLemma()");
			} elsif ($pos eq "U") {
				$pos = "0";
				push(@what, "FollowingVerbLemma");
				push(@word, &translate(\@l));
				push(@func, "");
			} else {
				push(@what, "Tag");
				push(@word, &translate(\@l));
				push(@func, "Lemma()");
			}
		} elsif ($c eq "P") {
			my $n = ord(shift(@l))-ord("0");
			push(@what, "Prefix$n");
			unshift(@l, join("", splice(@l, 0, $n)));
			push(@word, shift(@l));
			push(@func, "");
		} elsif ($c eq "S") {
			my $n = ord(shift(@l))-ord("0");
			push(@what, "Suffix$n");
			unshift(@l, join("", splice(@l, 0, $n)));
			push(@word, shift(@l));
			push(@func, "");
		} elsif ($c eq "I") {
			my $d = shift(@l);
			if ($d eq "1") {
				push(@what, "Num");
				push(@word, shift(@l));
				push(@func, "");
			} elsif ($d eq "A") {
				push(@what, "Cap");
				push(@word, shift(@l));
				push(@func, "");
			} elsif ($d eq "-") {
				push(@what, "Dash");
				push(@word, shift(@l));
				push(@func, "");
			} else {
				die("Unknown what: $c ($_)");
			}
		} elsif ($c eq "C") {
			push(@what, "Tag");
			push(@word, shift(@l));
			push(@func, "Substr(4,1)");
		} elsif ($c eq "G") {
			push(@what, "Tag");
			push(@word, shift(@l));
			push(@func, "Substr(2,1)");
		} elsif ($c eq "Q") {
			push(@what, "Tag", "Tag");
			if ($l[0] eq '-' and $l[1] eq '1') {
				push(@word, "NULL", "NULL");
				shift(@l);
				shift(@l);
			} else {
				push(@word, shift(@l), shift(@l));
			}
			push(@func, "Substr(1,1)", "Substr(4,1)");
		} elsif ($c eq "X") {
			my $d = shift(@l);
			push(@what, "Tag");
			push(@word, shift(@l));
			push(@func, "Substr($d,1)");
		} else {
			push(@what, "Tag");
			die("Unknown what: $c ($_)");
		}
		for (my $i = 0; $i < @word; $i++) {
			my $what = $what[$i];
			my $word = $word[$i];
			my $func = $func[$i];
			print "/" if ($i > 0);
			print "%[$pos,$what]";
			if ($func ne "") {
				print ".".$func;
			}
			if ($word ne "") {
				print "=$word";
			}
		}
	}
	print "\n";
}
