#!/usr/bin/env perl

use strict;
use Data::Dumper;

# covert Jan Raab's Morce feature template file into perc template file format.
# usage: morceft2percft.pl <morce.ft >perc.ft

(@ARGV == 0) or die("usage: morceft2percft.pl <morce.ft >perc.ft");

while (<STDIN>) {
	chomp;
	s/\r$//;
	my $first = 1;
	my @l = split(//);
	while (scalar(@l) > 0) {
		# position
		my $c = shift(@l);
		if ($first) {
			$first = 0;
		} else {
			print "/";
		}
		my $pos = "";
		if ($c eq "|") {
			$pos = "0";
		} elsif ($c eq "V") {
			$pos = "V";
		} elsif ($c eq "U") {
			$pos = "U";
		} elsif (ord($c) >= ord("A") and ord($c) <= ord("C")) {
			$pos = -1-ord($c)+ord("A");
		} elsif (ord($c) >= ord("a") and ord($c) <= ord("c")) {
			$pos = 1+ord($c)-ord("a");
		} else {
			die("Unknown position: $c ($_)");
		}
		# what
		$c = shift(@l);
		my @what = ();
		my @word = ();
		my @func = ();
		if ($c eq "T") {
			if ($pos eq "V") {
				$pos = "PreviousVerb()";
				push(@what, "Tag");
				shift @l;
				push(@func, "");
			} elsif ($pos eq "U") {
				$pos = "0";
				push(@what, "FollowingVerbTag");
				shift @l;
				push(@func, "");
			} else {
				push(@what, "Tag");
				shift @l;
				push(@func, "");
			}
		} elsif ($c eq "W") {
			push(@what, "Form");
			shift @l;
			push(@func, "");
		} elsif ($c eq "L") {
			if ($pos eq "V") {
				$pos = "PreviousVerb()";
				push(@what, "Lemma()");
				shift @l;
				push(@func, "");
			} elsif ($pos eq "U") {
				$pos = "0";
				push(@what, "FollowingVerbLemma");
				shift @l;
				push(@func, "");
			} else {
				push(@what, "Lemma()");
				shift @l;
				push(@func, "");
			}
		} elsif ($c eq "P") {
			my $n = ord(shift(@l))-ord("0");
			push(@what, "Prefix$n");
			splice(@l, 0, $n);
			push(@func, "");
		} elsif ($c eq "S") {
			my $n = ord(shift(@l))-ord("0");
			push(@what, "Suffix$n");
			splice(@l, 0, $n);
			push(@func, "");
		} elsif ($c eq "I") {
			my $d = shift(@l);
			if ($d eq "1") {
				push(@what, "Num");
				shift @l;
				push(@func, "");
			} elsif ($d eq "A") {
				push(@what, "Cap");
				shift @l;
				push(@func, "");
			} elsif ($d eq "-") {
				push(@what, "Dash");
				shift @l;
				push(@func, "");
			} else {
				die("Unknown what: $c ($_)");
			}
		} elsif ($c eq "C") {
			push(@what, "Tag");
			shift @l;
			push(@func, "Substr(4,1)");
		} elsif ($c eq "G") {
			push(@what, "Tag");
			shift @l;
			push(@func, "Substr(2,1)");
		} elsif ($c eq "Q") {
			push(@what, "Tag", "Tag");
			shift(@l);
			push(@func, "Substr(1,1)", "Substr(4,1)");
		} elsif ($c eq "X") {
			my $d = shift(@l);
			push(@what, "Tag");
			shift @l;
			push(@func, "Substr($d,1)");
		} elsif ($c eq "O") {
			push(@what, "Order");
			shift @l;
			push(@func, "");
		} else {
			die("Unknown what: $c ($_)");
		}
		for (my $i = 0; $i < @what; $i++) {
			my $what = $what[$i];
			my $func = $func[$i];
			print "/" if ($i > 0);
			print "%[$pos,$what]";
			if ($func ne "") {
				print ".".$func;
			}
		}
	}
	print "\n";
}
