#!/usr/bin/env bash

# Simple script for training Collins' averaged perceptron algorithm. Data files
# must be in perc format (UTF-8), best scoring data files (according to the
# test data) are saved using prefix.

TMP=/tmp

if [ $# -lt 5 ]; then
	echo "usage: train.sh [options] iterations feature_templates train_data dev_test_data prefix"
	exit 1;
fi

options=""
while [ $# -gt 5 ]; do
	options="$options $1"
	shift;
done

iters=$1
feature_templates=$2
train_data=$3
test_data=$4
prefix=$5

if which perc${TSFX} >/dev/null 2>&1; then
	base=
else
	base=`dirname $0`/../src/
fi

echo "Making features"
${base}perc${TSFX} make_features -t $feature_templates -F ${prefix}.f -D ${prefix}.dict "$train_data"
if [ $? -ne 0 ]; then
	exit 1;
fi

max=0
maxi=0
i=1
previ=0
while [ $i -le $iters ]; do
	if [ $previ -gt 0 ]; then
		ARG="-a ${TMP}/$$.alpha.$previ"
	fi
	${base}perc${TSFX} train $options -d ${prefix}.dict -f ${prefix}.f $ARG -A ${TMP}/$$.alpha.$i "$train_data"
	if [ $? -ne 0 ]; then
		exit 1;
	fi
	${base}perc${TSFX} test $options -d ${prefix}.dict -f ${prefix}.f -a ${TMP}/$$.alpha.$i "$test_data" 2>&1 >/dev/null|tee ${TMP}/$$.testout|grep -v "words processed, accuracy"
	if tail -n ${TMP}/$$.testout 2>/dev/null | grep accuracy >/dev/null; then
		exit 1;
	fi
	accuracy=`tail -n 1 ${TMP}/$$.testout | sed 's/.*accuracy: \([^ ]*\).*/\1/'`
	echo "Training iteration $i: $accuracy"
	rm -f ${TMP}/$$.testout

	if echo "$max-$accuracy"|bc|grep "^-" >/dev/null; then
		max=$accuracy
		maxi=$i
	fi
	previ=$i
	i=$(($i+1))
done

echo "Best iteration: $maxi ($max)"

mv ${TMP}/$$.alpha.$maxi "$prefix.alpha"
rm -f ${TMP}/$$.alpha.*
